@@ -5,7 +5,6 @@
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
-
 #include "desktopfileparser_p.h"
 
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
@@ -20,14 +19,19 @@
 #include <QJsonArray>
 #include <QJsonObject>
 #include <QMutex>
-#include <QStandardPaths>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
 // use if not else to prevent wrong scoping
-#define DESKTOPTOJSON_VERBOSE_DEBUG if (!DesktopFileParser::s_verbose) {} else qCDebug(DESKTOPPARSER)
-#define DESKTOPTOJSON_VERBOSE_WARNING if (!DesktopFileParser::s_verbose) {} else qCWarning(DESKTOPPARSER)
+#define DESKTOPTOJSON_VERBOSE_DEBUG                                                                                                                            \
+    if (!DesktopFileParser::s_verbose) {                                                                                                                       \
+    } else                                                                                                                                                     \
+        qCDebug(DESKTOPPARSER)
+#define DESKTOPTOJSON_VERBOSE_WARNING                                                                                                                          \
+    if (!DesktopFileParser::s_verbose) {                                                                                                                       \
+    } else                                                                                                                                                     \
+        qCWarning(DESKTOPPARSER)
 #else
 #define DESKTOPTOJSON_VERBOSE_DEBUG QT_NO_QDEBUG_MACRO()
 #define DESKTOPTOJSON_VERBOSE_WARNING QT_NO_QDEBUG_MACRO()
@@ -35,7 +39,6 @@
 
 #include "../lib/kcoreaddons_export.h"
 
-
 using namespace DesktopFileParser;
 
 // This code was taken from KConfigGroupPrivate::deserializeList
@@ -123,9 +126,15 @@ QByteArray DesktopFileParser::escapeValue(const QByteArray &input)
 
 struct CustomPropertyDefinition {
     // default ctor needed for QVector
-    CustomPropertyDefinition() : type(QVariant::String) {}
+    CustomPropertyDefinition()
+        : type(QVariant::String)
+    {
+    }
     CustomPropertyDefinition(const QByteArray &key, QVariant::Type type)
-        : key(key) , type(type) {}
+        : key(key)
+        , type(type)
+    {
+    }
     QJsonValue fromString(const QString &str) const
     {
         switch (type) {
@@ -168,8 +177,8 @@ struct CustomPropertyDefinition {
     QVariant::Type type;
 };
 
-namespace {
-
+namespace
+{
 bool readUntilDesktopEntryGroup(QFile &file, const QString &path, int &lineNr)
 {
     if (!file.open(QFile::ReadOnly)) {
@@ -188,7 +197,6 @@ bool readUntilDesktopEntryGroup(QFile &file, const QString &path, int &lineNr)
     return false;
 }
 
-
 QByteArray readTypeEntryForCurrentGroup(QFile &df, QByteArray *nextGroup, QByteArray *pName)
 {
     QByteArray group = *nextGroup;
@@ -212,14 +220,12 @@ QByteArray readTypeEntryForCurrentGroup(QFile &df, QByteArray *nextGroup, QByteA
             break;
         }
 
-        const static QRegularExpression typeEntryRegex(
-                QStringLiteral("^Type\\s*=\\s*(.*)$"));
+        const static QRegularExpression typeEntryRegex(QStringLiteral("^Type\\s*=\\s*(.*)$"));
         const auto match = typeEntryRegex.match(QString::fromUtf8(line));
         if (match.hasMatch()) {
             type = match.captured(1).toUtf8();
         } else if (pName) {
-            const static QRegularExpression nameEntryRegex(
-                    QStringLiteral("^X-KDE-ServiceType\\s*=\\s*(.*)$"));
+            const static QRegularExpression nameEntryRegex(QStringLiteral("^X-KDE-ServiceType\\s*=\\s*(.*)$"));
             const auto nameMatch = nameEntryRegex.match(QString::fromUtf8(line));
             if (nameMatch.hasMatch()) {
                 *pName = nameMatch.captured(1).toUtf8();
@@ -249,8 +255,10 @@ bool tokenizeKeyValue(QFile &df, const QString &src, QByteArray &key, QString &v
     // must have form key=value now
     const int equalsIndex = line.indexOf('=');
     if (equalsIndex == -1) {
-        qCWarning(DESKTOPPARSER).nospace() << qPrintable(src) << ':' << lineNr << ": Line is neither comment nor group "
-            "and doesn't contain an '=' character: \"" << line.constData() << '\"';
+        qCWarning(DESKTOPPARSER).nospace() << qPrintable(src) << ':' << lineNr
+                                           << ": Line is neither comment nor group "
+                                              "and doesn't contain an '=' character: \""
+                                           << line.constData() << '\"';
         return true;
     }
     // trim key and value to remove spaces around the '=' char
@@ -274,27 +282,35 @@ bool tokenizeKeyValue(QFile &df, const QString &src, QByteArray &key, QString &v
     return true;
 }
 
-static QString locateRelativeServiceType(const QString &relPath)
+#define KSERVICETYPES_SUBDIR "kservicetypes" QT_STRINGIFY(QT_VERSION_MAJOR) "/"
+
+static QString locateRelativeServiceType(const QString &relPath, const QStringList &serviceTypeSearchPaths)
 {
-    return QStandardPaths::locate(QStandardPaths::GenericDataLocation,
-                                  QStringLiteral("kservicetypes5/") + relPath);
+    for (const auto &dir : serviceTypeSearchPaths) {
+        QString path = dir + QLatin1String("/" KSERVICETYPES_SUBDIR) + relPath;
+        if (QFile::exists(path)) {
+            return path;
+        }
+    }
+    return QString();
 }
 
-static ServiceTypeDefinition* parseServiceTypesFile(const QString &inputPath)
+static ServiceTypeDefinition *parseServiceTypesFile(const QString &inputPath, const QStringList &serviceTypeSearchPaths)
 {
     int lineNr = 0;
     QString path = inputPath;
     if (QDir::isRelativePath(path)) {
-        path = locateRelativeServiceType(path);
+        path = locateRelativeServiceType(path, serviceTypeSearchPaths);
         QString rcPath;
         if (path.isEmpty()) {
-            rcPath = QLatin1String(":/kservicetypes5/") + inputPath;
+            rcPath = QLatin1String(":/" KSERVICETYPES_SUBDIR) + inputPath;
             if (QFileInfo::exists(rcPath)) {
                 path = rcPath;
             }
         }
         if (path.isEmpty()) {
-            qCWarning(DESKTOPPARSER).nospace() << "Could not locate service type file kservicetypes5/" << qPrintable(inputPath) << ", tried " << QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation) << " and " << rcPath;
+            qCWarning(DESKTOPPARSER).nospace() << "Could not locate service type file " << KSERVICETYPES_SUBDIR << qPrintable(inputPath) << ", tried "
+                                               << serviceTypeSearchPaths << " and " << rcPath;
             return nullptr;
         }
     }
@@ -312,15 +328,14 @@ static ServiceTypeDefinition* parseServiceTypesFile(const QString &inputPath)
     QByteArray nextGroup = "Desktop Entry";
     // Type must be ServiceType now
     QByteArray typeStr = readTypeEntryForCurrentGroup(df, &nextGroup, &result.m_serviceTypeName);
-    if (typeStr != QByteArrayLiteral("ServiceType")) {
-        qCWarning(DESKTOPPARSER) << path << "is not a valid service type: Type entry should be 'ServiceType', got"
-            << typeStr << "instead.";
+    if (typeStr != "ServiceType") {
+        qCWarning(DESKTOPPARSER) << path << "is not a valid service type: Type entry should be 'ServiceType', got" << typeStr << "instead.";
         return nullptr;
     }
     while (!df.atEnd()) {
         QByteArray currentGroup = nextGroup;
         typeStr = readTypeEntryForCurrentGroup(df, &nextGroup, nullptr);
-        if (!currentGroup.startsWith(QByteArrayLiteral("PropertyDef::"))) {
+        if (!currentGroup.startsWith("PropertyDef::")) {
             qCWarning(DESKTOPPARSER) << "Skipping invalid group" << currentGroup << "in service type" << path;
             continue;
         }
@@ -331,21 +346,23 @@ static ServiceTypeDefinition* parseServiceTypesFile(const QString &inputPath)
         QByteArray propertyName = currentGroup.mid(qstrlen("PropertyDef::"));
         QVariant::Type type = QVariant::nameToType(typeStr.constData());
         switch (type) {
-            case QVariant::String:
-            case QVariant::StringList:
-            case QVariant::Int:
-            case QVariant::Double:
-            case QVariant::Bool:
-                qCDebug(DESKTOPPARSER) << "Found property definition" << propertyName << "with type" << typeStr;
-                result.m_propertyDefs.push_back(CustomPropertyDefinition(propertyName, type));
-                break;
-            case QVariant::Invalid:
-                qCWarning(DESKTOPPARSER) << "Property type" << typeStr << "is not a known QVariant type."
-                        " Found while parsing property definition for" << propertyName << "in" << path;
-                break;
-            default:
-                qCWarning(DESKTOPPARSER) << "Unsupported property type" << typeStr << "for property" << propertyName
-                        << "found in" << path << "\nOnly QString, QStringList, int, double and bool are supported.";
+        case QVariant::String:
+        case QVariant::StringList:
+        case QVariant::Int:
+        case QVariant::Double:
+        case QVariant::Bool:
+            qCDebug(DESKTOPPARSER) << "Found property definition" << propertyName << "with type" << typeStr;
+            result.m_propertyDefs.push_back(CustomPropertyDefinition(propertyName, type));
+            break;
+        case QVariant::Invalid:
+            qCWarning(DESKTOPPARSER) << "Property type" << typeStr
+                                     << "is not a known QVariant type."
+                                        " Found while parsing property definition for"
+                                     << propertyName << "in" << path;
+            break;
+        default:
+            qCWarning(DESKTOPPARSER) << "Unsupported property type" << typeStr << "for property" << propertyName << "found in" << path
+                                     << "\nOnly QString, QStringList, int, double and bool are supported.";
         }
     }
     return new ServiceTypeDefinition(result);
@@ -358,14 +375,13 @@ Q_GLOBAL_STATIC(ServiceTypesHash, s_serviceTypes)
 QBasicMutex s_serviceTypesMutex;
 } // end of anonymous namespace
 
-
-ServiceTypeDefinitions ServiceTypeDefinitions::fromFiles(const QStringList &paths)
+ServiceTypeDefinitions ServiceTypeDefinitions::fromFiles(const QStringList &paths, const QStringList &serviceTypesSearchPaths)
 {
     ServiceTypeDefinitions ret;
     ret.m_definitions.reserve(paths.size());
     // as we might modify the cache we need to acquire a mutex here
     for (const QString &serviceTypePath : paths) {
-        bool added = ret.addFile(serviceTypePath);
+        bool added = ret.addFile(serviceTypePath, serviceTypesSearchPaths);
         if (!added) {
 #ifdef BUILDING_DESKTOPTOJSON_TOOL
             exit(1); // this is a fatal error when using kcoreaddons_desktop_to_json()
@@ -375,10 +391,10 @@ ServiceTypeDefinitions ServiceTypeDefinitions::fromFiles(const QStringList &path
     return ret;
 }
 
-bool ServiceTypeDefinitions::addFile(const QString& path)
+bool ServiceTypeDefinitions::addFile(const QString &path, const QStringList &serviceTypesSearchPaths)
 {
     QMutexLocker lock(&s_serviceTypesMutex);
-    ServiceTypeDefinition* def = s_serviceTypes->object(path);
+    ServiceTypeDefinition *def = s_serviceTypes->object(path);
 
     if (def) {
         // in cache but we still must make our own copy
@@ -386,7 +402,7 @@ bool ServiceTypeDefinitions::addFile(const QString& path)
     } else {
         // not found in cache -> we need to parse the file
         qCDebug(DESKTOPPARSER) << "About to parse service type file" << path;
-        def = parseServiceTypesFile(path);
+        def = parseServiceTypesFile(path, serviceTypesSearchPaths);
         if (!def) {
             return false;
         }
@@ -397,14 +413,27 @@ bool ServiceTypeDefinitions::addFile(const QString& path)
     return true;
 }
 
-QJsonValue ServiceTypeDefinitions::parseValue(const QByteArray &key, const QString &value) const
+QJsonValue ServiceTypeDefinitions::parseValue(const QByteArray &aKey, const QString &value) const
 {
+    QByteArray key = aKey;
+    // Remove translation markers "X-KDE-Keywords[en_GB]"
+    const int startIdx = key.indexOf('[');
+    if (startIdx != -1) {
+        Q_ASSERT(key.endsWith(']'));
+        key.truncate(startIdx);
+    }
+
+    auto findKey = [&key](const ServiceTypeDefinition &def) {
+        return std::find_if(def.m_propertyDefs.cbegin(), def.m_propertyDefs.cend(), [&key](const CustomPropertyDefinition &propertyDef) {
+            return propertyDef.key == key;
+        });
+    };
+
     // check whether the key has a special type associated with it
     for (const auto &def : m_definitions) {
-        for (const CustomPropertyDefinition &propertyDef : def.m_propertyDefs) {
-            if (propertyDef.key == key) {
-                return propertyDef.fromString(value);
-            }
+        auto it = findKey(def);
+        if (it != def.m_propertyDefs.cend()) {
+            return it->fromString(value);
         }
     }
     qCDebug(DESKTOPPARSER) << "Unknown property type for key" << key << "-> falling back to string";
@@ -414,13 +443,17 @@ QJsonValue ServiceTypeDefinitions::parseValue(const QByteArray &key, const QStri
 bool ServiceTypeDefinitions::hasServiceType(const QByteArray &serviceTypeName) const
 {
     const auto it = std::find_if(m_definitions.begin(), m_definitions.end(), [&serviceTypeName](const ServiceTypeDefinition &def) {
-            return def.m_serviceTypeName == serviceTypeName;
+        return def.m_serviceTypeName == serviceTypeName;
     });
     return it != m_definitions.end();
 }
 
-void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefinitions &serviceTypes, const QString &value,
-                                      QJsonObject &json, QJsonObject &kplugin, int lineNr)
+void DesktopFileParser::convertToJson(const QByteArray &key,
+                                      ServiceTypeDefinitions &serviceTypes,
+                                      const QString &value,
+                                      QJsonObject &json,
+                                      QJsonObject &kplugin,
+                                      int lineNr)
 {
     /* The following keys are recognized (and added to a "KPlugin" object):
 
@@ -446,69 +479,69 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
         X-KDE-PluginInfo-EnabledByDefault=true
         X-KDE-FormFactors=desktop
     */
-    if (key == QByteArrayLiteral("Icon")) {
+    if (key == "Icon") {
         kplugin[QStringLiteral("Icon")] = value;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Name")) {
+    } else if (key == "X-KDE-PluginInfo-Name") {
         kplugin[QStringLiteral("Id")] = value;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Category")) {
+    } else if (key == "X-KDE-PluginInfo-Category") {
         kplugin[QStringLiteral("Category")] = value;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-License")) {
+    } else if (key == "X-KDE-PluginInfo-License") {
         kplugin[QStringLiteral("License")] = value;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Copyright")) {
+    } else if (key == "X-KDE-PluginInfo-Copyright") {
         kplugin[QStringLiteral("Copyright")] = value;
-    } else if (key.startsWith(QByteArrayLiteral("X-KDE-PluginInfo-Copyright["))) {
+    } else if (key.startsWith("X-KDE-PluginInfo-Copyright[")) {
         const QString languageSuffix = QString::fromUtf8(key.mid(qstrlen("X-KDE-PluginInfo-Copyright")));
         kplugin[QStringLiteral("Copyright") + languageSuffix] = value;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Version")) {
+    } else if (key == "X-KDE-PluginInfo-Version") {
         kplugin[QStringLiteral("Version")] = value;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Website")) {
+    } else if (key == "X-KDE-PluginInfo-Website") {
         kplugin[QStringLiteral("Website")] = value;
     }
 #if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 79)
-    else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Depends")) {
+    else if (key == "X-KDE-PluginInfo-Depends") {
         kplugin[QStringLiteral("Dependencies")] = QJsonArray::fromStringList(deserializeList(value));
         qCDebug(DESKTOPPARSER) << "The X-KDE-PluginInfo-Depends property is deprecated and will be removed in KF6";
     }
 #endif
-    else if (key == QByteArrayLiteral("X-KDE-ServiceTypes") || key == QByteArrayLiteral("ServiceTypes")) {
-        //NOTE: "X-KDE-ServiceTypes" and "ServiceTypes" were already managed in the first parse step, so this second one is almost a noop
+    else if (key == "X-KDE-ServiceTypes" || key == "ServiceTypes") {
+        // NOTE: "X-KDE-ServiceTypes" and "ServiceTypes" were already managed in the first parse step, so this second one is almost a noop
         const auto services = deserializeList(value);
         kplugin[QStringLiteral("ServiceTypes")] = QJsonArray::fromStringList(services);
-    } else if (key == QByteArrayLiteral("MimeType")) {
+    } else if (key == "MimeType") {
         // MimeType is a XDG string list and not a KConfig list so we need to use ';' as the separator
         kplugin[QStringLiteral("MimeTypes")] = QJsonArray::fromStringList(deserializeList(value, ';'));
         // make sure that applications using kcoreaddons_desktop_to_json() that depend on reading
         // the MimeType property still work (see https://git.reviewboard.kde.org/r/125527/)
         json[QStringLiteral("MimeType")] = value; // TODO KF6 remove this compatibility code
-    } else if (key == QByteArrayLiteral("X-KDE-FormFactors")) {
+    } else if (key == "X-KDE-FormFactors") {
         kplugin[QStringLiteral("FormFactors")] = QJsonArray::fromStringList(deserializeList(value));
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-EnabledByDefault")) {
+    } else if (key == "X-KDE-PluginInfo-EnabledByDefault") {
         bool boolValue = false;
         // should only be lower case, but be tolerant here
         if (value.toLower() == QLatin1String("true")) {
             boolValue = true;
         } else {
             if (value.toLower() != QLatin1String("false")) {
-                qCWarning(DESKTOPPARSER).nospace() << "Expected boolean value for key \"" << key
-                    << "\" at line " << lineNr << "but got \"" << value << "\" instead.";
+                qCWarning(DESKTOPPARSER).nospace() << "Expected boolean value for key \"" << key << "\" at line " << lineNr << "but got \"" << value
+                                                   << "\" instead.";
             }
         }
         kplugin[QStringLiteral("EnabledByDefault")] = boolValue;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Author")) {
+    } else if (key == "X-KDE-PluginInfo-Author") {
         QJsonObject authorsObject = kplugin.value(QStringLiteral("Authors")).toArray().at(0).toObject();
         // if the authors object doesn't exist yet this will create it
         authorsObject[QStringLiteral("Name")] = value;
         QJsonArray array;
         array.append(authorsObject);
         kplugin[QStringLiteral("Authors")] = array;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Email")) {
+    } else if (key == "X-KDE-PluginInfo-Email") {
         QJsonObject authorsObject = kplugin.value(QStringLiteral("Authors")).toArray().at(0).toObject();
         // if the authors object doesn't exist yet this will create it
         authorsObject[QStringLiteral("Email")] = value;
         QJsonArray array;
         array.append(authorsObject);
         kplugin[QStringLiteral("Authors")] = array;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Authors")) {
+    } else if (key == "X-KDE-PluginInfo-Authors") {
         const auto authors = deserializeList(value);
         QJsonArray oldArray = kplugin.value(QStringLiteral("Authors")).toArray();
         QJsonArray newArray;
@@ -519,7 +552,7 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
             newArray.append(authorsObject);
         }
         kplugin[QStringLiteral("Authors")] = newArray;
-    } else if (key == QByteArrayLiteral("X-KDE-PluginInfo-Emails")) {
+    } else if (key == "X-KDE-PluginInfo-Emails") {
         const auto emails = deserializeList(value);
         QJsonArray oldArray = kplugin.value(QStringLiteral("Authors")).toArray();
         QJsonArray newArray;
@@ -530,24 +563,20 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
             newArray.append(authorsObject);
         }
         kplugin[QStringLiteral("Authors")] = newArray;
-    } else if (key == QByteArrayLiteral("Name") || key.startsWith(QByteArrayLiteral("Name["))) {
+    } else if (key == "Name" || key.startsWith("Name[")) {
         // TODO: also handle GenericName? does that make any sense, or is X-KDE-PluginInfo-Category enough?
         kplugin[QString::fromUtf8(key)] = value;
-    } else if (key == QByteArrayLiteral("Comment")) {
+    } else if (key == "Comment") {
         kplugin[QStringLiteral("Description")] = value;
-    } else if (key.startsWith(QByteArrayLiteral("Comment["))) {
+    } else if (key.startsWith("Comment[")) {
         kplugin[QStringLiteral("Description") + QString::fromUtf8(key.mid(qstrlen("Comment")))] = value;
-    } else if (key == QByteArrayLiteral("InitialPreference")) {
+    } else if (key == "InitialPreference") {
         kplugin[QStringLiteral("InitialPreference")] = value.toInt();
-    } else if (key == QByteArrayLiteral("Hidden")) {
+    } else if (key == "Hidden") {
         DESKTOPTOJSON_VERBOSE_WARNING << "Hidden= key found in desktop file, this makes no sense"
-                " with metadata inside the plugin.";
+                                         " with metadata inside the plugin.";
         kplugin[QString::fromUtf8(key)] = (value.toLower() == QLatin1String("true"));
-    } else if (key == QByteArrayLiteral("Exec") ||
-               key == QByteArrayLiteral("Type") ||
-               key == QByteArrayLiteral("Actions") ||
-               key == QByteArrayLiteral("X-KDE-Library") ||
-               key == QByteArrayLiteral("Encoding")) {
+    } else if (key == "Exec" || key == "Type" || key == "Actions" || key == "X-KDE-Library" || key == "Encoding") {
         // Exec= doesn't make sense here, however some .desktop files (like e.g. in kdevelop) have a dummy value here
         // also the Type=Service entry is no longer needed
         // Actions= is used as hack at least with the Dolphin KPart to report the different view mode options
@@ -561,16 +590,20 @@ void DesktopFileParser::convertToJson(const QByteArray &key, ServiceTypeDefiniti
     }
 }
 
-bool DesktopFileParser::convert(const QString &src, const QStringList &serviceTypes, QJsonObject &json, QString *libraryPath)
+bool DesktopFileParser::convert(const QString &src,
+                                const QStringList &serviceTypes,
+                                QJsonObject &json,
+                                QString *libraryPath,
+                                const QStringList &serviceTypesSearchPaths)
 {
     QFile df(src);
     int lineNr = 0;
-    ServiceTypeDefinitions serviceTypeDef = ServiceTypeDefinitions::fromFiles(serviceTypes);
+    ServiceTypeDefinitions serviceTypeDef = ServiceTypeDefinitions::fromFiles(serviceTypes, serviceTypesSearchPaths);
     readUntilDesktopEntryGroup(df, src, lineNr);
     DESKTOPTOJSON_VERBOSE_DEBUG << "Found [Desktop Entry] group in line" << lineNr;
     auto startPos = df.pos();
 
-    //parse it a first time to know servicetype
+    // parse it a first time to know servicetype
     while (!df.atEnd()) {
         QByteArray key;
         QString value;
@@ -578,7 +611,7 @@ bool DesktopFileParser::convert(const QString &src, const QStringList &serviceTy
             break;
         }
         // some .desktop files still use the legacy ServiceTypes= key
-        if (key == QByteArrayLiteral("X-KDE-ServiceTypes") || key == QByteArrayLiteral("ServiceTypes")) {
+        if (key == "X-KDE-ServiceTypes" || key == "ServiceTypes") {
             const QString dotDesktop = QStringLiteral(".desktop");
             const QChar slashChar(QLatin1Char('/'));
             const auto serviceList = deserializeList(value);
@@ -586,23 +619,23 @@ bool DesktopFileParser::convert(const QString &src, const QStringList &serviceTy
             for (const auto &service : serviceList) {
                 if (!serviceTypeDef.hasServiceType(service.toLatin1())) {
                     // Make up the filename from the service type name. This assumes consistent naming...
-                    QString absFileName = locateRelativeServiceType(
-                            service.toLower().replace(slashChar, QLatin1Char('-')) + dotDesktop);
+                    QString absFileName =
+                        locateRelativeServiceType(service.toLower().replace(slashChar, QLatin1Char('-')) + dotDesktop, serviceTypesSearchPaths);
                     if (absFileName.isEmpty()) {
-                        absFileName = locateRelativeServiceType(
-                                service.toLower().remove(slashChar) + dotDesktop);
+                        absFileName = locateRelativeServiceType(service.toLower().remove(slashChar) + dotDesktop, serviceTypesSearchPaths);
                     }
                     if (absFileName.isEmpty()) {
-                        qCWarning(DESKTOPPARSER) << "Unable to find service type for service" << service << "listed in" << src;
+                        qCWarning(DESKTOPPARSER) << "Unable to find service type for service" << service << "listed in" << src
+                                                 << "- service type search directories were" << serviceTypesSearchPaths;
                     } else {
-                        serviceTypeDef.addFile(absFileName);
+                        serviceTypeDef.addFile(absFileName, serviceTypesSearchPaths);
                     }
                 }
             }
             break;
         }
     }
-    lineNr=0;
+    lineNr = 0;
     df.seek(startPos);
 
     QJsonObject kplugin; // the "KPlugin" key of the metadata
@@ -623,11 +656,10 @@ bool DesktopFileParser::convert(const QString &src, const QStringList &serviceTy
 #else
         convertToJson(key, serviceTypeDef, value, json, kplugin, lineNr);
 #endif
-        if (libraryPath && key == QByteArrayLiteral("X-KDE-Library")) {
+        if (libraryPath && key == "X-KDE-Library") {
             *libraryPath = value;
         }
     }
     json[QStringLiteral("KPlugin")] = kplugin;
     return true;
 }
-
