@@ -10,6 +10,7 @@
 
 #include <QString>
 #include <QStringView>
+#include <QVector>
 
 /**
  * Custom toLower function which is much faster than
@@ -31,6 +32,7 @@ static bool match_recursive(QStringView::const_iterator pattern,
                             const uint8_t *srcMatches,
                             uint8_t *matches,
                             int nextMatch,
+                            int &totalMatches,
                             int &recursionCount)
 {
     static constexpr int recursionLimit = 10;
@@ -78,7 +80,7 @@ static bool match_recursive(QStringView::const_iterator pattern,
             const auto strNextChar = std::next(str);
             if (!matchingInSequence && match_recursive(pattern, strNextChar, recursiveScore, strBegin,
                                 strEnd, patternEnd, matches, recursiveMatches,
-                                nextMatch, recursionCount)) {
+                                nextMatch, totalMatches, recursionCount)) {
                 // Pick best recursive score
                 if (!recursiveMatch || recursiveScore > bestRecursiveScore) {
                     memcpy(bestRecursiveMatches, recursiveMatches, maxMatches);
@@ -102,7 +104,7 @@ static bool match_recursive(QStringView::const_iterator pattern,
 
     // Calculate score
     if (matched) {
-        static constexpr int sequentialBonus = 30;
+        static constexpr int sequentialBonus = 25;
         static constexpr int separatorBonus = 25; // bonus if match occurs after a separator
         static constexpr int camelBonus = 25; // bonus if match is uppercase and prev is lower
         static constexpr int firstLetterBonus = 15; // bonus if the first letter is matched
@@ -169,6 +171,8 @@ static bool match_recursive(QStringView::const_iterator pattern,
         }
     }
 
+    totalMatches = nextMatch;
+
     // Return best result
     if (recursiveMatch && (!matched || bestRecursiveScore > outScore)) {
         // Recursive score is better than "this"
@@ -198,7 +202,8 @@ static bool match_internal(QStringView pattern, QStringView str, int &outScore,
     const auto patternEnd = pattern.cend();
     const auto strEnd = str.cend();
 
-    return match_recursive(patternIt, strIt, outScore, strIt, strEnd, patternEnd, nullptr, matches, 0, recursionCount);
+    int total = 0;
+    return match_recursive(patternIt, strIt, outScore, strIt, strEnd, patternEnd, nullptr, matches, 0, total, recursionCount);
 }
 
 /**************************************************************/
@@ -206,21 +211,94 @@ static bool match_internal(QStringView pattern, QStringView str, int &outScore,
 bool KFuzzyMatcher::matchSimple(QStringView pattern, QStringView str)
 {
     auto patternIt = pattern.cbegin();
+    /**
+     * Instead of doing
+     *
+     *      strIt.toLower() == patternIt.toLower()
+     *
+     * we convert patternIt to Upper / Lower as needed and compare with strIt. This
+     * saves us from calling toLower() on both strings, making things a little bit faster
+     */
+    bool lower = patternIt->isLower();
+    QChar cUp = lower ? patternIt->toUpper() : *patternIt;
+    QChar cLow = lower ? *patternIt : patternIt->toLower();
     for (auto strIt = str.cbegin(); strIt != str.cend() && patternIt != pattern.cend(); ++strIt) {
-        if (strIt->toLower() == patternIt->toLower()) {
+        if (*strIt == cLow || *strIt == cUp) {
             ++patternIt;
+            lower = patternIt->isLower();
+            cUp = lower ? patternIt->toUpper() : *patternIt;
+            cLow = lower ? *patternIt : patternIt->toLower();
         }
     }
+
     return patternIt == pattern.cend();
 }
 
 KFuzzyMatcher::Result KFuzzyMatcher::match(QStringView pattern, QStringView str)
 {
-    uint8_t matches[256];
-    int score = 0;
-    const bool matched = match_internal(pattern, str, score, matches);
+    /**
+     * Simple substring matching to flush out non-matching strings
+     */
+    const bool simpleMatch = matchSimple(pattern, str);
+
     KFuzzyMatcher::Result result;
+    result.matched = false;
+    result.score = 0;
+
+    if (!simpleMatch) {
+        return result;
+    }
+
+    // actual algorithm
+    int score = 0;
+    uint8_t matches[256];
+    const bool matched = match_internal(pattern, str, score, matches);
     result.matched = matched;
     result.score = score;
     return result;
 }
+
+QVector<KFuzzyMatcher::Range> KFuzzyMatcher::matchedRanges(QStringView pattern, QStringView str, RangeType type)
+{
+    QVector<KFuzzyMatcher::Range> ranges;
+    if (pattern.isEmpty()) {
+        return ranges;
+    }
+
+    int totalMatches = 0;
+    int score = 0;
+    int recursionCount = 0;
+
+    auto strIt = str.cbegin();
+    auto patternIt = pattern.cbegin();
+    const auto patternEnd = pattern.cend();
+    const auto strEnd = str.cend();
+
+    uint8_t matches[256];
+    auto res = match_recursive(patternIt, strIt, score, strIt, strEnd, patternEnd, nullptr, matches, 0, totalMatches, recursionCount);
+    // didn't match? => We don't care about results
+    if (!res && type == RangeType::FullyMatched) {
+        return {};
+    }
+
+    int previousMatch = 0;
+    for (int i = 0; i < totalMatches; ++i) {
+        auto matchPos = matches[i];
+        /**
+         * Check if this match is part of the previous
+         * match. If it is, we increase the length of
+         * the last range.
+         */
+        if (!ranges.isEmpty() && matchPos == previousMatch + 1) {
+            ranges.last().length++;
+        } else {
+            /**
+             * This is a new match inside the string
+             */
+            ranges.push_back({/* start: */ matchPos, /* length: */ 1});
+        }
+        previousMatch = matchPos;
+    }
+
+    return ranges;
+}
