@@ -17,17 +17,15 @@
  */
 namespace KTextToHTML
 {
-
 /**
  * @see Options
  * @since 5.5.0
  */
-enum Option
-{
+enum Option {
     /**
      * Preserve white-space formatting of the text
      */
-    PreserveSpaces  = 1 << 1,
+    PreserveSpaces = 1 << 1,
 
     /**
      * Replace text emoticons smileys by emoticons images.
@@ -36,24 +34,24 @@ enum Option
      * This option works only when KEmoticons framework is available at runtime,
      * and requires QGuiApplication, otherwise the flag is simply ignored.
      */
-    ReplaceSmileys  = 1 << 2,
+    ReplaceSmileys = 1 << 2,
 
     /**
      * Don't parse and replace any URLs.
      */
-    IgnoreUrls      = 1 << 3,
+    IgnoreUrls = 1 << 3,
 
     /**
      * Interpret text highlighting markup, like *bold*, _underline_ and /italic/,
      * and wrap them in corresponding HTML entities.
      */
-    HighlightText   = 1 << 4,
+    HighlightText = 1 << 4,
 
     /**
      * Replace phone numbers with tel: links.
      * @since 5.56.0
      */
-    ConvertPhoneNumbers = 1 << 5
+    ConvertPhoneNumbers = 1 << 5,
 };
 /**
  * Stores a combination of #Option values.
@@ -80,10 +78,7 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(Options)
  *
  * @since 5.5.0
  */
-KCOREADDONS_EXPORT QString convertToHtml(const QString &plainText,
-                                         const KTextToHTML::Options &options,
-                                         int maxUrlLen = 4096,
-                                         int maxAddressLen = 255);
+KCOREADDONS_EXPORT QString convertToHtml(const QString &plainText, const KTextToHTML::Options &options, int maxUrlLen = 4096, int maxAddressLen = 255);
 
 }
 
